@@ -10,7 +10,7 @@
 
 using namespace Mata::Nfa;
 using namespace Mata::Strings;
-using namespace Mata::util;
+using namespace Mata::Util;
 using namespace Mata::Parser;
 using namespace Mata::RE2Parser;
 
@@ -368,16 +368,18 @@ TEST_CASE("Mata::Nfa::SegNfa::noodlify_for_equation() both sides") {
         create_nfa(&z, "(a|b)*");
         create_nfa(&w, "(a|b)*");
 
-        auto res = std::vector<std::vector<std::pair<Nfa, SegNfa::EpsCntVector>>>( { 
-                {{x, {0, 0} }, {x, {0, 1} }, {y, {1, 1} }}, 
+        auto res = std::vector<std::vector<std::pair<Nfa, SegNfa::EpsCntVector>>>( {
+                {{x, {0, 0} }, {x, {0, 1} }, {y, {1, 1} }},
                 {{x, {0, 0} }, {y, {1, 0} }, {y, {1, 1} }} } );
         SegNfa::NoodleSubstSequence noodles = SegNfa::noodlify_for_equation(
-            std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(x), std::make_shared<Nfa>(y) }, 
+            std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(x), std::make_shared<Nfa>(y) },
             std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(z), std::make_shared<Nfa>(w)});
         for(size_t i = 0; i < noodles.size(); i++) {
             for(size_t j = 0; j < noodles[i].size(); j++) {
                 CHECK(noodles[i][j].second == res[i][j].second);
                 CHECK(are_equivalent(*noodles[i][j].first.get(), res[i][j].first, nullptr));
+                auto used_symbols{ noodles[i][j].first->get_used_symbols() };
+                CHECK(used_symbols.find(EPSILON) == used_symbols.end());
             }
         }
     }
@@ -390,19 +392,21 @@ TEST_CASE("Mata::Nfa::SegNfa::noodlify_for_equation() both sides") {
         create_nfa(&w, "(a|b)+");
         create_nfa(&astar, "a*");
 
-        auto res = std::vector<std::vector<std::pair<Nfa, SegNfa::EpsCntVector>>>( { 
-                {{x, {0, 1} }, {z, {1, 1} }}, 
-                {{x, {0, 0} }, {w, {1, 1} }}, 
-                {{x, {0, 0} }, {x, {0, 1} }, {z, {1, 1} }}, 
-                {{x, {0, 0} }, {z, {1, 0} }, {w, {1, 1} }}, 
+        auto res = std::vector<std::vector<std::pair<Nfa, SegNfa::EpsCntVector>>>( {
+                {{x, {0, 1} }, {z, {1, 1} }},
+                {{x, {0, 0} }, {w, {1, 1} }},
+                {{x, {0, 0} }, {x, {0, 1} }, {z, {1, 1} }},
+                {{x, {0, 0} }, {z, {1, 0} }, {w, {1, 1} }},
          } );
         SegNfa::NoodleSubstSequence noodles = SegNfa::noodlify_for_equation(
-            std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(x), std::make_shared<Nfa>(y) }, 
+            std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(x), std::make_shared<Nfa>(y) },
             std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(z), std::make_shared<Nfa>(w)});
         for(size_t i = 0; i < noodles.size(); i++) {
             for(size_t j = 0; j < noodles[i].size(); j++) {
                 CHECK(noodles[i][j].second == res[i][j].second);
                 CHECK(are_equivalent(*noodles[i][j].first.get(), res[i][j].first, nullptr));
+                auto used_symbols{ noodles[i][j].first->get_used_symbols() };
+                CHECK(used_symbols.find(EPSILON) == used_symbols.end());
             }
         }
     }
@@ -416,13 +420,15 @@ TEST_CASE("Mata::Nfa::SegNfa::noodlify_for_equation() both sides") {
 
         auto res = std::vector<std::vector<std::pair<Nfa, SegNfa::EpsCntVector>>>( {} );
        SegNfa::NoodleSubstSequence noodles = SegNfa::noodlify_for_equation(
-            std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(x) }, 
+            std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(x) },
             std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(y), std::make_shared<Nfa>(z), std::make_shared<Nfa>(w)});
         CHECK(noodles.size() == 1);
         for(size_t i = 0; i < noodles.size(); i++) {
             for(size_t j = 0; j < noodles[i].size(); j++) {
                 CHECK(noodles[i][j].second == res[i][j].second);
                 CHECK(are_equivalent(*noodles[i][j].first.get(), res[i][j].first, nullptr));
+                auto used_symbols{ noodles[i][j].first->get_used_symbols() };
+                CHECK(used_symbols.find(EPSILON) == used_symbols.end());
             }
         }
     }
@@ -435,17 +441,43 @@ TEST_CASE("Mata::Nfa::SegNfa::noodlify_for_equation() both sides") {
         create_nfa(&w, "(a|b)*");
 
         auto res = std::vector<std::vector<std::pair<Nfa, SegNfa::EpsCntVector>>>( {
-                {{y, {1, 1} }}, 
+                {{y, {1, 1} }},
                 {{y, {1, 0} }, {y, {1, 1} }},
             } );
         SegNfa::NoodleSubstSequence noodles = SegNfa::noodlify_for_equation(
-            std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(x), std::make_shared<Nfa>(y) }, 
+            std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(x), std::make_shared<Nfa>(y) },
             std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(z), std::make_shared<Nfa>(w)});
         CHECK(noodles.size() == 2);
         for(size_t i = 0; i < noodles.size(); i++) {
             for(size_t j = 0; j < noodles[i].size(); j++) {
                 CHECK(noodles[i][j].second == res[i][j].second);
                 CHECK(are_equivalent(*noodles[i][j].first.get(), res[i][j].first, nullptr));
+                auto used_symbols{ noodles[i][j].first->get_used_symbols() };
+                CHECK(used_symbols.find(EPSILON) == used_symbols.end());
+            }
+        }
+    }
+
+    SECTION("Simple automata -- regex 1") {
+        Nfa x, y, z, u;
+        create_nfa(&x, "a");
+        create_nfa(&y, "ab*");
+        create_nfa(&z, "ab*");
+        create_nfa(&u, "a*");
+
+        auto res = std::vector<std::vector<std::pair<Nfa, SegNfa::EpsCntVector>>>( {
+                {{x, {0, 0} }, {x, {1, 1} }},
+            } );
+        SegNfa::NoodleSubstSequence noodles = SegNfa::noodlify_for_equation(
+            std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(x), std::make_shared<Nfa>(y) },
+            std::vector<std::shared_ptr<Nfa>>{std::make_shared<Nfa>(z), std::make_shared<Nfa>(u)});
+        CHECK(noodles.size() == 1);
+        for(size_t i = 0; i < noodles.size(); i++) {
+            for(size_t j = 0; j < noodles[i].size(); j++) {
+                CHECK(noodles[i][j].second == res[i][j].second);
+                CHECK(are_equivalent(*noodles[i][j].first.get(), res[i][j].first, nullptr));
+                auto used_symbols{ noodles[i][j].first->get_used_symbols() };
+                CHECK(used_symbols.find(EPSILON) == used_symbols.end());
             }
         }
     }
